//
//  Controller.m
//  UTModLauncher
//
//  Created by David McGavern.
//  Copyright (c) 2004 David McGavern, Loopware. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//

#import "Controller.h"

#import "NSStringCategory.h"
@implementation Controller

- (void)awakeFromNib{
    //We're setting up the main window for the ber fading action:
    [window setAlphaValue:0];
    [window center];
    
    //The array of mods (each "mod" is an NSDictionary)
    mods=[[NSMutableArray alloc] init];
    
    //The UT mod! Arguably the best one of them all ;-)
    NSDictionary *defaultDict=[NSDictionary dictionaryWithObjects:
	[NSArray arrayWithObjects:@"Unreal Tournament 2004", @"", @"http://www.unrealtournament.com", nil]
	forKeys:
	[NSArray arrayWithObjects:@"name", @"command", @"url", nil]];
    [mods addObject:defaultDict];
    
    //Set default prefs:
    NSMutableDictionary *defaultPrefs=[NSMutableDictionary dictionary];
    [defaultPrefs setObject:[NSNumber numberWithBool:YES] forKey:@"dvdCheck"];
    [defaultPrefs setObject:[NSNumber numberWithBool:YES] forKey:@"fadingWindows"];
    [preferences registerDefaults:defaultPrefs];
    
    //Setup URL button in the url column...
    NSTableColumn *tableColumn = [tableView tableColumnWithIdentifier: @"url"];    
    NSButtonCell *buttonCell=[[[NSButtonCell alloc] init] autorelease];
    [buttonCell setTitle:@""];
    [buttonCell setImagePosition:NSImageOnly];
    [buttonCell setBezelStyle:NSRegularSquareBezelStyle];
    [buttonCell setButtonType:NSMomentaryChangeButton];
    [buttonCell setImage:[NSImage imageNamed:@"url"]];
    [buttonCell setAlternateImage:[NSImage imageNamed:@"urlPressed"]];
    [buttonCell setBordered:NO];
    [buttonCell setAction:@selector(openUrl:)];
    [tableColumn setDataCell:buttonCell];
    
    //Probably should be named something else, but this method searches
    //for the UT installation. If it can't find it, it asks the user.
    [self setUnrealLocation];
    
    //We need to find the actual UT mods sometime, and now seems perfect!
    //This will be detached as a new thread, and will write home once it
    //finds any mods.
    [NSThread detachNewThreadSelector:@selector(findAllUnrealMods) toTarget:self withObject:nil];
    
    //Show the window (it will fade)...
    [window makeKeyAndOrderFront:self];
}

- (IBAction)launch:(id)sender
{
    //Called when the "Launch" button is pressed

    //We're making sure that the UT file is valid...
    if (![[NSFileManager defaultManager] fileExistsAtPath:[[preferences objectForKey:@"unrealPath"] stringByAppendingString:@"/Contents/MacOS/Unreal Tournament 2004"]]){
	//Aww, it isn't valid :'(. Now we need to find UT again...
	[preferences setObject:nil forKey:@"unrealPath"];
	
	[self setUnrealLocation];
    }
    
    //The DVD check in action. Very basic, but we're not MacSoft :-D
    //It's a convenience thing, it's easier to click OK in UTModLauncher
    //then having to launch UT first.
    if (![[NSFileManager defaultManager] fileExistsAtPath:@"/Volumes/Unreal Tournament 2004 for MacOS X"] && [preferences boolForKey:@"dvdCheck"]){
	    NSBeginAlertSheet(
		NSLocalizedString(@"Please insert the UT2004 DVD...", nil),  // sheet message
		NSLocalizedString(@"OK", nil),              // default button label
		nil,                    // third button
		nil,              // other button label
		window,                 // window sheet is attached to
		self,                   // well be our own delegate
		nil,
					// did-end selector
		NULL,                   // no need for did-dismiss selector
		sender,                 // context info
		NSLocalizedString(@"The Unreal Tournament 2004 DVD must be inserted for UT2004 to launch. Please insert the DVD and click the \"Launch\" button again.\n\nIf you would like to disable this DVD check, you may do so from the UTModLauncher menu. Note that this will not disable UT's check, just UTModLauncher's.", nil),
					// additional text
		nil);                   // no parameters in message
	    return;
    }

    [statusField setStringValue:@"Launching, please wait..."];
    [busyIndicator startAnimation:self];
    
    [launchButton setEnabled:NO];
    //When locked, the table doesn't change and the launch button doesn't work.
    locked=YES;
    
    //We love our threads! :-D This will launch the game.
    [NSThread detachNewThreadSelector:@selector(launchThread) toTarget:self withObject:nil];
}

- (void)launchThread{
    //This is a thread, we need our own autorelease pool...
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    
    //launchName is taken from the mod folder's name
    NSString *launchName=[[mods objectAtIndex:[tableView selectedRow]] objectForKey:@"launchName"];
    NSString *argument=@"";
    //Our launch argument
    if (launchName) argument=[NSString stringWithFormat:@"-mod=%@", launchName];
    
    //If the mod has a custom command defined, we're gonna use it!
    NSString *customCommand=[[mods objectAtIndex:[tableView selectedRow]] objectForKey:@"command"];
    NSMutableArray *arguments=[NSMutableArray array];
    
    //Doing various stuff to customCommand
    customCommand=[NSString stringByReplacing:@"\n" with:@"" inString:[NSString stringByDeletingLeadSpaceInString:customCommand]];
    customCommand=[NSString stringByReplacing:@"\r" with:@"" inString:customCommand];
        
    //Add the arguments we'll be using when launching UT
    if ([customCommand length]>0) [arguments addObject:customCommand]; //Only add if there's a custom command
    [arguments addObject:argument];

    //Setting up an NSTask to launch UT...
    NSTask *launcher = [[NSTask alloc] init];
    [launcher setCurrentDirectoryPath:[[preferences objectForKey:@"unrealPath"] stringByAppendingString:@""]];
    [launcher setLaunchPath:[[preferences objectForKey:@"unrealPath"] stringByAppendingString:@"/Contents/MacOS/Unreal Tournament 2004"]];
    [launcher setArguments:arguments];
    [launcher launch];
    
    //We're taking a small break so people get to see our lovely
    //progress indicator while UT's launching!:-D
    system("sleep 5");
    //Close the window after we've awoken again
    [window performSelectorOnMainThread:@selector(close) withObject:nil waitUntilDone:NO];
    [self performSelectorOnMainThread:@selector(stopIndicator) withObject:nil waitUntilDone:NO];
    
    //Wait until UT has closed.. then continue
    [launcher waitUntilExit];
    
    [self performSelectorOnMainThread:@selector(utClosed) withObject:nil waitUntilDone:NO];
    
    [pool release];
}

- (void)stopIndicator{
    [busyIndicator stopAnimation:self];
}

- (void)utClosed{
    [window makeKeyAndOrderFront:self];
    [statusField setStringValue:@"Welcome back!"];
    [statusField performSelector:@selector(setStringValue:) withObject:@"" afterDelay:3];
    if ([tableView selectedRow]>-1) [launchButton setEnabled:YES];
    
    locked=NO;
}
- (BOOL)validateMenuItem:(NSMenuItem *)anItem{
    //Validates the menu items, in this case we only need one check
    //for our DVD check item.
    if ([[anItem title] isEqualToString:NSLocalizedString(@"Enable DVD Check", nil)]){
	if ([preferences boolForKey:@"dvdCheck"]){
	    [anItem setState:NSOnState];
	} else {
	    [anItem setState:NSOffState];
	}
    }
    return YES;
}
- (IBAction)toggleDVDCheck:(id)sender{
    if ([preferences boolForKey:@"dvdCheck"]){
	[preferences setBool:NO forKey:@"dvdCheck"];
    } else {
	[preferences setBool:YES forKey:@"dvdCheck"];
    }
}
#pragma mark FINDING STUFF

- (void)newUnrealModWasFound:(NSDictionary *)newMod{
    //Called by the worker thread when a new mod has been found
    [mods addObject:newMod];
    
    [tableView reloadData];
}


- (void)modSearchBegan{
    //Called by worker thread when search begins
    searching=YES;
    
    [busyIndicator startAnimation:self];
    [statusField setStringValue:@"Searching for mods..."];
    
    locked=YES;
}
- (void)modSearchEnded{
    //Ditto as above, except when it ends
    searching=NO;
    
    [busyIndicator stopAnimation:self];
    [statusField setStringValue:@""];
    
    [self tableViewSelectionDidChange:nil];
    
    locked=NO;
    //NSLog ([mods description]);
}

- (void)findAllUnrealMods{
    //Begins finding all mods, in a separate thread (of course)
    
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

    [self performSelectorOnMainThread:@selector(modSearchBegan) withObject:nil waitUntilDone:NO];
    
    NSString *basePath = [preferences objectForKey:@"unrealPath"];
    
    [self searchForModsInBasePath:basePath];
    
    //The following is commented out since mods in the App support directory
    //don't really seem to work well :-(
    //[self searchForModsInBasePath:[@"~/Library/Application Support/Unreal Tournament 2004/" stringByExpandingTildeInPath]];
        
    [self performSelectorOnMainThread:@selector(modSearchEnded) withObject:nil waitUntilDone:NO];
    
    [pool release];
}

- (void)searchForModsInBasePath:(NSString *)path{
    //The main worker method! Looks for qualified,
    //yummy UT mods in path.

    int i;
    NSString *file;
    NSArray *array;
    NSMutableDictionary *newEntry;
    NSString *pluginPath=[path stringByAppendingString:@"/"];

    if (pluginPath){
	// This the directory you will be searching
	NSDirectoryEnumerator *directoryEnumerator = [[NSFileManager defaultManager] enumeratorAtPath:pluginPath];
	// this is the enumerator that will search through the files for you
	NSString *pathName = [[NSString alloc] init];
	NSString *currentLine;
	// this is just use to compare the extension
	while (pathName = [directoryEnumerator nextObject]) {
	    // keeps giving your the name from each file
	    if ([[[pathName lastPathComponent] lowercaseString] isEqualToString:@"ut2k4mod.ini"]) {
		file=[NSString stringWithContentsOfFile:[pluginPath stringByAppendingString:pathName]];		
		
		array=[file componentsSeparatedByString:@"\n"];
		if ([array count]==1) //Do we need to use Macintosh line breaks?
		    array=[file componentsSeparatedByString:@"\r"];
		
		newEntry=[NSMutableDictionary dictionary];
		
		for (i=0; i<[array count]; i++){
		    currentLine=[[array objectAtIndex:i] lowercaseString];
		    //Check for mod title
		    if ([currentLine hasPrefix:@"modtitle="]){
			[newEntry setObject:[NSString stringByDeletingLeadSpaceInString:[NSString stringByReplacing:@"\"" with:@"" inString:[[array objectAtIndex:i] substringFromIndex:9]]]forKey:@"name"];
		    } else if ([currentLine hasPrefix:@"modcmdline="]){
			[newEntry setObject:[NSString stringByDeletingLeadSpaceInString:[NSString stringByReplacing:@"\"" with:@"" inString:[[array objectAtIndex:i] substringFromIndex:11]]] forKey:@"command"];
		    } else if ([currentLine hasPrefix:@"modurl="]){
			[newEntry setObject:[NSString stringByDeletingLeadSpaceInString:[NSString stringByReplacing:@"\"" with:@"" inString:[[array objectAtIndex:i] substringFromIndex:7]]] forKey:@"url"];
		    }
		}
		
		[newEntry setObject:[[pathName pathComponents] objectAtIndex:0] forKey:@"launchName"];
		
		[self performSelectorOnMainThread:@selector(newUnrealModWasFound:) withObject:newEntry waitUntilDone:NO];
		
	    }
	}
    
    }

}

- (BOOL)panel:(id)sender isValidFilename:(NSString *)filename{
    if ([[filename lastPathComponent] isEqualToString:@"Unreal Tournament 2004.app"]){
	return YES;
    }
    
    NSBeep ();
    return NO;
}

- (void)setUnrealLocation
{
    int i, line=-1;
    //Find the location of UT from the log file...
    NSString *inputFile = [[NSString alloc] initWithContentsOfFile:[[NSString stringWithString:@"~/Library/Application Support/Unreal Tournament 2004/System/UT2004.log"] stringByExpandingTildeInPath]];
    
    if (!inputFile && ![preferences objectForKey:@"unrealPath"]) {
	//We didn't find a log file... so... ummm.... uhhh.... ask the user!!
	NSOpenPanel *panel=[NSOpenPanel openPanel];
	[panel setTitle:@"Select your copy of UT2k4"];
	[panel setPrompt:@"Select"];
	[panel setDelegate:self];
	//Only accept UT2k4, NO FAKING!
	[panel runModalForDirectory:@"/Applications" file:@"Unreal Tournament 2004.app" types:[NSArray arrayWithObject:@"app"]];
	
	//Only write it to prefs if they actually selected UT2k4
	if ([panel filename]){
	    [preferences setObject:[panel filename] forKey:@"unrealPath"];
	} else {
	    //Buh bye! Don't let the door hit you on the way out ;-)
	    [NSApp terminate:self];
	}
	
	return;
    }
    
    if ([preferences objectForKey:@"unrealPath"]){
	return;
    }
    
    //Separate the file by linebreaks and put the result into the array
    NSArray *tempArray=[inputFile componentsSeparatedByString:@"\n"];
    if ([tempArray count]==0) //Do we need to use Macintosh line breaks?
	tempArray=[inputFile componentsSeparatedByString:@"\r"];
    
    // Now searching for line that starts with Init:
    for (i=0; i<[tempArray count]; i++){
	NSString *temp=[tempArray objectAtIndex:i];
	if ([temp length]>=22){
	    if ([[[tempArray objectAtIndex:i] substringWithRange:NSMakeRange(0,22)] isEqualToString:@"Init: Base directory: "]){
		//YAY! we found the line number!
		line=i;
	    }
	}
    }
    //The line that has the path on it...
    NSString *lineString=[tempArray objectAtIndex:line];

    lineString = [lineString substringFromIndex:22];
    lineString = [lineString substringWithRange:NSMakeRange(0, [lineString length]-7)];
    
    [preferences setObject:lineString forKey:@"unrealPath"];
        
}

#pragma mark TABLE VIEW
- (BOOL)tableView:(NSTableView *)aTableView shouldSelectRow:(int)rowIndex{
    return (!locked);
}

- (void)tableView:(NSTableView*)sender willDisplayCell:(NSCell*)cell 
forTableColumn:(NSTableColumn*)column row:(int)row{
    if ([[column identifier] isEqualToString:@"url"]){
	if ([(NSString *)[[mods objectAtIndex:row] objectForKey:@"url"] length]<5){ 
	    [cell setImage:nil]; 
	    [(NSButtonCell *)cell setAlternateImage:nil]; 
	    return;
	}
	
	[(NSButtonCell *)cell setAlternateImage:[NSImage imageNamed:@"urlPressed"]];
	
	if (row==[tableView selectedRow] && [window isKeyWindow] && [window firstResponder]==tableView){
	    [cell setImage:[NSImage imageNamed:@"urlSelected"]];
	} else {
	    [cell setImage:[NSImage imageNamed:@"url"]];
	}
    } 
}

- (IBAction)openUrl:(id)sender{
    if (locked) return;

    NSString *url=[[mods objectAtIndex:[tableView selectedRow]] objectForKey:@"url"];
    if ([url length]>6){ 
	//Check for no http:// in the string (BAD RedOrchestra developers ;-)
	if (![url hasPrefix:@"http://"]) url=[@"http://" stringByAppendingString:url];
	
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:url]];
	
	NSLog (url);
    }
}

- (id)tableView:(NSTableView *)aTableView
    objectValueForTableColumn:(NSTableColumn *)aTableColumn
    row:(int)rowIndex {
    
    return [[mods objectAtIndex:rowIndex] objectForKey:[aTableColumn identifier]];
    
}

- (void)tableView:(NSTableView *)aTableView
    setObjectValue:anObject
    forTableColumn:(NSTableColumn *)aTableColumn
    row:(int)rowIndex {
    
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView {
    return [mods count];
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification{
    if (!searching){
	if ([tableView selectedRow]>-1){
	    [launchButton setEnabled:YES];
	} else {
	    [launchButton setEnabled:NO];
	}
    }
}

@end

/////////////////
//   THE END   //
/////////////////
